@@ -2,6 +2,8 @@
 import sys,time
 import re
 import os
+import urllib
+import urllib2
 
 from subprocess import Popen,PIPE
 
@@ -9,6 +11,30 @@ from PyQt4 import QtGui,QtCore
 from PyQt4.QtGui import *
 from pyPianobar_ui import *
 
+class retrieveAlbum(QtCore.QThread):
+	def __init__(self):
+		self.iUrl = ""
+		QtCore.QThread.__init__(self)
+
+	def conf (self,artist,album):
+		album = album.replace("(Radio Single)","")
+		album = album.replace("(Explicit)","")
+		album = re.sub("[^A-Za-z\s]","",album.rstrip().lstrip())
+		data = { 'artist': artist, 'album': album }
+		self.url_value = urllib.urlencode(data)
+	
+	def run(self):
+		__pyqtSignals__ = ( "album" )
+		url = 'http://www.slothradio.com/covers?' + self.url_value
+		self.response = urllib2.urlopen(url) 
+		page = self.response.read()
+		datastart = page.find('<div class="album0"><img src="', 0) + 30
+		if datastart > 30:
+			dataend = page.find('"',datastart)
+			self.iUrl = page[datastart:dataend]
+			self.emit(QtCore.SIGNAL("album"))
+		self.response.close
+
 class Config:
 	def __init__(self):
 		self.user = ""
@@ -94,6 +120,21 @@ class Config:
 	def get_var(self,var,line):
 		r = re.match("^"+var+"=(.+)$",line)
 		return r.group(1)
+
+class snooze(QtCore.QThread):
+	__pyqtSignals__ = ( "shutdown" )
+
+	def __init__(self,time):
+		self.time = time
+		QtCore.QThread.__init__(self)
+		self.cont = 1
+	
+	def run(self):
+		time.sleep(self.time*60)
+		if self.cont:
+			self.emit(QtCore.SIGNAL("shutdown"))
+			
+			
 		
 class tPandora(QtCore.QThread):
 
@@ -192,12 +233,14 @@ class MyForm(QtGui.QMainWindow):
 	def __init__(self, parent=None):
 		QtGui.QWidget.__init__(self, parent)
 		self.station = 0;
-		self.ui = Ui_MainWindow()
+		self.ui = Ui_pyPianobar()
 		self.ui.setupUi(self)
 		self.c = Config()
 		self.c.load()
+		self.s = snooze(0)
 		self.p = tPandora(self.c)
 		self.p.start()
+		self.a = retrieveAlbum()
 		
 		QtCore.QObject.connect(self.p, QtCore.SIGNAL('newsong'), self.newsong)
 		QtCore.QObject.connect(self.p, QtCore.SIGNAL('stations'), self.stations)
@@ -205,19 +248,20 @@ class MyForm(QtGui.QMainWindow):
 		QtCore.QObject.connect(self.p, QtCore.SIGNAL('showUsername'), self.showUsername)
 		QtCore.QObject.connect(self.p, QtCore.SIGNAL('showPassword'), self.showPassword)
 		QtCore.QObject.connect(self.p, QtCore.SIGNAL('second'), self.uSecond)
+		QtCore.QObject.connect(self.s, QtCore.SIGNAL('shutdown'), self.sQuit)
+		QtCore.QObject.connect(self.a, QtCore.SIGNAL('album'), self.uAlbum)
 		QtCore.QObject.connect(self.ui.btnLove, QtCore.SIGNAL('clicked()'), self.doLove)
 		QtCore.QObject.connect(self.ui.btnHate, QtCore.SIGNAL('clicked()'), self.doHate)
 		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('clicked()'), self.doNext)
 		QtCore.QObject.connect(self.ui.btnPlay, QtCore.SIGNAL('clicked()'), self.doPlayPause)
-		QtCore.QObject.connect(self.ui.btnSleep, QtCore.SIGNAL('clicked()'), self.sQuit)
+		QtCore.QObject.connect(self.ui.btnSleep, QtCore.SIGNAL('clicked()'), self.tiredSong)
 		QtCore.QObject.connect(self.ui.actionUser_Pass, QtCore.SIGNAL('triggered()'), self.setUserPass)
 		QtCore.QObject.connect(self.ui.actionLastfm_User_Pass, QtCore.SIGNAL('triggered()'), self.setLastfm)
 		QtCore.QObject.connect(self.ui.actionDelete, QtCore.SIGNAL('triggered()'), self.deleteStation)
 		QtCore.QObject.connect(self.ui.actionCreate, QtCore.SIGNAL('triggered()'), self.createStation)
-		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('pressed()'), self.nextActive)
-		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('released()'), self.nextInactive)
-		QtCore.QObject.connect(self.ui.listStations, QtCore.SIGNAL('itemDoubleClicked(QListWidgetItem *)'), self.setStation)
-		QtCore.QObject.connect(self.ui.centralwidget, QtCore.SIGNAL('destroyed()'), self.sQuit)
+		QtCore.QObject.connect(self.ui.actionSnooze, QtCore.SIGNAL('triggered()'), self.setSnooze)
+		QtCore.QObject.connect(self.ui.listStations, QtCore.SIGNAL('activated(int)'), self.setStation)
+		QtCore.QObject.connect(self, QtCore.SIGNAL('triggered()'), self.closeEvent)
 
 	def setUserPass(self):
 		self.showUsername()
@@ -231,78 +275,101 @@ class MyForm(QtGui.QMainWindow):
 			elif self.p.last == "song":
 				self.p.pandora.stdin.write(self.c.act_stationchange)
 				self.p.pandora.stdin.write("\n")
+				self.ui.btnPlay.setStyleSheet("background-color: #FFFFFF;\nbackground-image: url(:/img/img/paused-trans.png);")
+				self.p.playing = 0
 
 			self.p.pandora.stdin.write(self.c.act_stationcreate)
 			self.p.pandora.stdin.write(station+"\n")
+			self.station = 0
 			what = QtGui.QMessageBox.question(self, 'Artist or Track?', 'Is this an artist or track name?','Artist','Track')
 			if what == 0:
 				self.p.pandora.stdin.write("a")
 			else:
 				self.p.pandora.stdin.write("t")
 
-			ok = QtGui.QMessageBox.warning(self,'Select Match','Double click the closest title from the station list.')
+			ok = QtGui.QMessageBox.warning(self,'Select Match','Select the closest title from the station drop down.')
 			self.p.last = 'create'
 
 	def deleteStation(self):
 		reply = QtGui.QMessageBox.question(self, 'Delete Station',
-			"Really delete "+self.ui.listStations.currentItem().text()+"?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+			"Really delete "+self.ui.listStations.currentText()+"?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
 		if reply == QtGui.QMessageBox.Yes:
-			if self.p.last == "stations":
-				self.p.pandora.stdin.write(str(self.ui.listStations.currentRow())+"\n")
-				self.p.pandora.stdin.write(self.c.act_stationdelete)
-				self.p.pandora.stdin.write("y\n")
-				self.p.pandora.stdin.write(self.c.act_stationchange)
-			elif self.p.last == "song":
-				self.p.pandora.stdin.write(self.c.act_stationchange)
-				self.p.pandora.stdin.write(str(self.ui.listStations.currentRow())+"\n")
-				self.p.pandora.stdin.write(self.c.act_stationdelete)
-				self.p.pandora.stdin.write("y\n")
+			if self.p.last == "song":
 				self.p.pandora.stdin.write(self.c.act_stationchange)
+				self.ui.btnPlay.setStyleSheet("background-color: #FFFFFF;\nbackground-image: url(:/img/img/paused-trans.png);")
+				self.p.playing = 0
+			self.p.pandora.stdin.write(str(self.ui.listStations.currentIndex())+"\n")
+			self.p.pandora.stdin.write(self.c.act_stationdelete)
+			self.p.pandora.stdin.write("y\n")
+			self.p.pandora.stdin.write(self.c.act_stationchange)
+			self.station = 0
 	
 	def stations(self):
 		self.ui.listStations.clear()
 		for station in self.p.stations:
 			self.ui.listStations.addItem(station)
-			self.ui.listStations.setCurrentRow(self.station)
+			self.ui.listStations.setCurrentIndex(self.station)
 
 	def setStation(self, listaItem):
+		self.ui.btnNext.setFocus()
 		if self.p.last == 'create':
-			self.p.pandora.stdin.write(str(self.ui.listStations.currentRow())+"\n")
-			self.p.pandora.stdin.write(self.c.act_stationchange)
+			self.p.pandora.stdin.write(str(self.ui.listStations.currentIndex())+"\n")
 			QtGui.QMessageBox.warning(self, 'Station Created', 'Your station has been created.')
+			self.p.pandora.stdin.write(self.c.act_stationchange)
 		else:
 			if self.p.last == "song":
 				self.p.pandora.stdin.write(self.c.act_stationchange)
 		
-			self.station = self.ui.listStations.currentRow()
-			self.p.pandora.stdin.write(str(self.ui.listStations.currentRow())+"\n")
-			self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/playing.png);")
+			self.station = self.ui.listStations.currentIndex()
+			self.p.pandora.stdin.write(str(self.ui.listStations.currentIndex())+"\n")
+			self.ui.btnPlay.setStyleSheet("background-color: #FFFFFF;\nbackground-image: url(:/img/img/playing-trans.png);")
+
+	def tiredSong(self):
+		if self.p.last == "song":
+			self.p.pandora.stdin.write(self.c.act_songtired)
+	
+	def setSnooze(self):
+		time,ok = QtGui.QInputDialog.getText(self,"Snooze", "Enter a time, in minutes, to shutdown:")
+		if ok:
+			self.s.time = int(time)
+			self.s.start()
 
 	def uSecond(self):
 		self.ui.lblTime.setText(self.p.length)
 
-	def nextActive(self):
-		self.ui.btnNext.setStyleSheet("background-image: url(:/img/img/skip-active.png);")
-
-	def nextInactive(self):
-		self.ui.btnNext.setStyleSheet("background-image: url(:/img/img/skip.png);")
-
 	def newsong(self):
-		self.ui.lblSong.setText(self.p.sTitle)
-		self.ui.lblArtist.setText("by: " + self.p.sArtist)
+		self.setWindowTitle(QtGui.QApplication.translate("pyPianobar", "\"" + self.p.sTitle + "\"" + " by " + self.p.sArtist + " - pyPianobar", None, QtGui.QApplication.UnicodeUTF8))
+		self.ui.albumArt.setHtml("")
 		self.ui.lblAlbum.setText("on: " + self.p.sAlbum)
+		self.a.conf (self.p.sArtist,self.p.sAlbum)
+		self.a.start()
 		if self.p.sLike:
-			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/plus-active.png);")
+			self.ui.btnLove.setStyleSheet("background-color: rgb(255, 255, 255);\n"
+			"background-image: url(:/img/img/thumbs-up-trans.png);")
+			self.setLoveDown()
 		else:
-			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/plus.png);")
+			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/thumbs-up-trans.png);\nbackground-color: #000000;")
+
 		if not self.p.playing:
-			self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/playing.png);")
+			self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/playing-trans.png);\nbackground-color: #FFFFFF;")
+			self.setPlayDown()
 			self.p.playing = 1
+
+	def uAlbum(self):
+		myUrl = QtCore.QUrl.fromEncoded(self.a.iUrl)
+		self.ui.albumArt.load(myUrl)
 		
 	def doLove(self):
 		if self.p.last == "song":
 	 		self.p.pandora.stdin.write(self.c.act_songlove)
-			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/plus-active.png);")
+			self.ui.btnLove.setStyleSheet("background-color: rgb(255, 255, 255);\n"
+			"background-image: url(:/img/img/thumbs-up-trans.png);")
+			self.ui.btnNext.setFocus()
+			self.setLoveDown()
+			self.setPlayDown()
+	
+	def setLoveDown(self):
+		self.ui.btnLove.setDown ( 1 )
 		
 	def doHate(self):
 		if self.p.last == "song":
@@ -312,17 +379,24 @@ class MyForm(QtGui.QMainWindow):
 		if self.p.last == "song":
 			self.p.pandora.stdin.write(self.c.act_songnext)
 
+	def setPlayDown(self):
+		self.ui.btnPlay.setDown ( 1 )
+
 	def doPlayPause(self):
 		if self.p.last == "stations":
-			self.p.pandora.stdin.write(str(self.ui.listStations.currentRow())+"\n")
+			self.p.pandora.stdin.write(str(self.ui.listStations.currentIndex())+"\n")
 		else:
 			self.p.pandora.stdin.write(self.c.act_songpause)
 
 		if self.p.playing:
-			self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/paused.png);")
+			self.ui.btnPlay.setStyleSheet("background-color: #FFFFFF;\nbackground-image: url(:/img/img/paused-trans.png);")
+			self.ui.btnNext.setFocus()
+			self.setPlayDown()
 			self.p.playing = 0
 		else:
-			self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/playing.png);")
+			self.ui.btnPlay.setStyleSheet("background-color: #FFFFFF;\nbackground-image: url(:/img/img/playing-trans.png);")
+			self.ui.btnNext.setFocus()
+			self.setPlayDown()
 			self.p.playing = 1
 	
 	def sQuit(self):
@@ -332,9 +406,20 @@ class MyForm(QtGui.QMainWindow):
 		self.p.exit()
 		self.doQuit()
 	
+	def closeEvent(self, event):
+		self.p.drun = 0
+		self.p.pandora.stdin.write('\n'+self.c.act_quit)
+		self.p.pandora.wait()
+		self.p.exit()
+		self.doQuit()
+
+	
 	def doError(self):
 		if self.p.crash < 2:
 			ok = QtGui.QMessageBox.warning( self, "Pianobar Error", "Pianobar experienced an error.\nPianobar Output:\n" + self.p.error + "\nRestarting Pianobar.", QtGui.QMessageBox.Ok )
+			if re.match("^Username and/or password not correct.+",self.p.error) is not None:
+				self.showUsername()
+				self.showPassword()
 			self.p.crash += 1
 			self.p.drun = 1
 			self.p.start()
@@ -362,7 +447,8 @@ class MyForm(QtGui.QMainWindow):
 		username,ok = QtGui.QInputDialog.getText(self,"Username", "Pandora Username:")
 		if ok:
 			self.c.user = username
-			self.p.pandora.stdin.write(self.c.user+"\n")
+			if self.p.last == "username":
+				self.p.pandora.stdin.write(self.c.user+"\n")
 		else:
 			self.p.drun = 0
 			self.p.exit()
@@ -372,7 +458,8 @@ class MyForm(QtGui.QMainWindow):
 		password,ok = QtGui.QInputDialog.getText(self,"Password", "Pandora Password:",QLineEdit.Password)
 		if ok:
 			self.c.password = password
-			self.p.pandora.stdin.write(self.c.password+"\n")
+			if self.p.last == "password" and not self.p.pandora.stdin.closed:
+				self.p.pandora.stdin.write(self.c.password+"\n")
 			self.c.write()
 		else:
 			self.p.drun = 0
